<?php

declare(strict_types=1);

namespace Drupal\Tests\dark_mode_toggle\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;

/**
 * Tests the JavaScript functionality of the Dark Mode Toggle module.
 *
 * @group dark_mode_toggle
 */
final class DarkModeToggleTest extends WebDriverTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'block',
    'user',
    'dark_mode_toggle',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $admin_user = $this->drupalCreateUser([
      'administer site configuration',
    ]);

    // Log in as the admin user, create/place the dark mode toggle block and
    // visit the front page.
    $this->drupalLogin($admin_user);
    $this->drupalPlaceBlock('dark_mode_toggle');
    $this->drupalGet('<front>');
  }

  /**
   * Tests whether the dark mode functionality is loaded correctly.
   */
  public function testJavaScriptFilesAreLoadedCorrectly(): void {
    $assert_session = $this->assertSession();

    // To prevent FOUC, the initialization script should be in the head element.
    $assert_session->elementExists('css', 'head script[src*="dark-mode-toggle.init.js"]');

    // The additional script should be loaded in the body element.
    $assert_session->elementExists('css', 'body script[src*="dark-mode-toggle.js"]');

    // Verify the data-dark-mode-source attribute is set and initially has the
    // value 'system'.
    $assert_session->elementAttributeContains('css', 'html', 'data-dark-mode-source', 'system');

    // Ensure once() is applied.
    $assert_session->elementAttributeContains('css', 'html', 'data-once', 'darkModeToggle');
  }

  /**
   * Tests whether the dark mode toggle button works correctly.
   */
  public function testDarkModeToggleButtons(): void {
    $assert_session = $this->assertSession();

    $this->drupalGet('<front>');

    // Verify that all dark mode buttons are present.
    $assert_session->buttonExists('Light');
    $assert_session->buttonExists('Dark');
    $assert_session->buttonExists('System');
  }

}
